#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Titanium Mobile UI Builder plugin
#

import os, sys, shutil, codecs, re
from xml.dom import minidom

ignoreFiles = ['.gitignore', '.cvsignore', '.DS_Store']
ignoreDirs = ['.git','.svn','_svn', 'CVS']

i = 0
js = [ '', '' ]
jss = ''

def build(node, level):
	global i, js, jss
	if not level:
		i = 0
		js = [ '', '' ]
		jss = ''
	indent = '  '
	base = ''
	params = []
	events = []
	text = re.sub(r'\n', r'\\n', node.firstChild.wholeText.strip()) if node.firstChild else ''
	for item in node.attributes.values():
		if item.nodeName == 'js:base':
			base = item.nodeValue
		elif item.nodeName == 'base':
			base = '"' + item.nodeValue + '"'
		elif item.nodeName == 'js:class':
			params.append('"' + item.nodeName[3:] + '":' + item.nodeValue)
		elif item.nodeName == 'class':
			params.append('"' + item.nodeName + '":"' + item.nodeValue + '"')
		elif item.nodeName.startswith('on:'):
			events.append(item.nodeName[3:] + ':"' + item.nodeValue + '"')
		elif item.nodeName.startswith('js:') and item.nodeValue == 'L(nodeText)':
			params.append(item.nodeName[3:] + ':L("' + text + '")')
		elif item.nodeName.startswith('js:') and item.nodeValue == 'nodeText':
			params.append(item.nodeName[3:] + ':"' + text + '"')
		elif item.nodeName.startswith('js:'):
			params.append(item.nodeName[3:] + ':' + item.nodeValue)
		else:
			params.append(item.nodeName + ':"' + item.nodeValue + '"')
	nodes = []
	for item in node.childNodes:
		if item.nodeType != node.ELEMENT_NODE: continue
		if item.tagName == 'Style':
			jss += '\n' + item.firstChild.wholeText.strip() + '\n\n' if item.firstChild.wholeText.strip() else ''
		elif item.tagName == 'Script':
			js[i] += "\nTi.include('" + item.getAttribute('src') + "');\n" if item.getAttribute('src') else ''
			js[i] += '\n' + item.firstChild.wholeText.strip() + '\n' if item.firstChild.wholeText.strip() else ''
		else:
			i = 1
			nodes.append(build(item, level + 1))
	if level:
		ui  = '{ui:"' + node.tagName + '"'
		ui += ', base:' + base if base else ''
		ui += ', params:{' + ', '.join(params) + '}' if params else ''
		ui += ', events:{' + ', '.join(events) + '}' if events else ''
		ui += ', children:[\n' + indent * level + (',\n' + indent * level).join(nodes) + '\n' + indent * (level - 1) + ']' if nodes else ''
		ui += '}'
	else:
		ui  = "var TUB = require('lib/tub');\n\n"
		ui += js[0]
		for item in nodes: ui += 'TUB.buildUI(\n' + item + '\n);\n'
		ui += js[1]
	return ui

def compile(config):
	global jss
	print '[INFO] Titanium User i/f Builder plugin loaded and working for %s' % os.path.basename(config['project_dir'])
	src_dir = os.path.join(config['project_dir'],'Pages')
	dst_dir = os.path.join(config['project_dir'],'Resources')

	for root, dirs, files in os.walk(src_dir):
		for dir in dirs:
			if dir in ignoreDirs:
				dirs.remove(dir)
		if len(files) > 0:
			prefix = root[len(src_dir):]
			for file in files:
				fp = os.path.splitext(file)
				if len(fp) != 2: continue
				if file in ignoreFiles: continue
				path = os.path.join(root, file)
				dest = os.path.join(dst_dir, path[len(src_dir):].strip('/\\'))
				if not os.path.exists(os.path.dirname(dest)):
					os.makedirs(os.path.dirname(dest))
				if fp[1] == '.xml':
					dest = os.path.splitext(dest)[0] + '.js'
					if os.path.exists(dest) and os.path.getmtime(dest) >= max(os.path.getmtime(path), os.path.getmtime(__file__)): continue
					print 'CONVERTING FILE: %s => %s' % (path, dest)
					js = build(minidom.parse(path).documentElement, 0)
					if js:
						dst = codecs.open(dest, 'wb', encoding = 'utf-8')
						dst.write(js)
						dst.close()
#					elif os.path.exists(dest):
#						os.unlink(dest)
					dest = os.path.splitext(dest)[0] + '.jss'
					if jss:
						dst = open(dest, 'wb')
						dst.write(jss)
						dst.close()
					elif os.path.exists(dest):
						os.unlink(dest)
				elif not os.path.exists(dest) or os.path.getmtime(path) > os.path.getmtime(dest):
					print 'COPYING FILE: %s => %s' % (path, dest)
					shutil.copy(path, dest)

if __name__ == '__main__':
	if len(sys.argv) != 2 or sys.argv[1]=='--help':
		print "Usage: %s <project_directory>" % os.path.basename(sys.argv[0])
		sys.exit(1)
	compile({'project_dir':os.path.abspath(sys.argv[1])})
